/*
 * Copyright (c) 2004, 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-conf-1.c,v 1.11 2005/06/20 19:03:15 ca Exp $")

#if SM_LIBCONF_ALONE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sm-conf.h"
#else /* SM_LIBCONF_ALONE */
#include "sm/string.h"
#include "sm/net.h"
#include "sm/io.h"
#include "sm/sm-conf.h"
#include "sm/sm-conf-prt.h"
#define SM_CONF_LOG_DEF 1
#include "sm/logcnfdef.h"
#include <stdio.h>
#endif /* SM_LIBCONF_ALONE */

/*
**  Just a test program to play around with libconf.
*/

typedef struct
{
	char const	*s_socket;
	ipv4_T		 s_ipv4;
	uint32_t	 s_flags;
	unsigned int	 s_int_u32;
	unsigned int	 s_type;
	unsigned short	 s_limit;	/* between 2 and 12 */
	unsigned long	 s_duration;
	uchar		 s_char;
	sm_logspec_T	 s_log;
} structure;

sm_conf_definition_T const flag_names[] =
{
{ SM_CONF_DEF_MAGIC, "Flag1",	sm_conf_type_choice_value, 0x01	},
{ SM_CONF_DEF_MAGIC, "Flag2",	sm_conf_type_choice_value, 0x02 },

/* Sentinel */
{ SM_CONF_DEF_MAGIC, NULL }
};


sm_conf_definition_T const alias_options[] =
{
{ SM_CONF_DEF_MAGIC, "none",		sm_conf_type_choice_value, 0x00	},
{ SM_CONF_DEF_MAGIC, "localpart",	sm_conf_type_choice_value, 0x04 },
{ SM_CONF_DEF_MAGIC, "localdomains",	sm_conf_type_choice_value, 0x08 },
{ SM_CONF_DEF_MAGIC, "all",		sm_conf_type_choice_value, 0x10 },

/* Sentinel */
{ SM_CONF_DEF_MAGIC, NULL }
};


sm_conf_definition_T const limit_constraints[] =
{
	{ SM_CONF_DEF_MAGIC, "", sm_conf_type_u32_minimum, 2	},
	{ SM_CONF_DEF_MAGIC, "", sm_conf_type_u32_maximum, 12	},

	/* Sentinel */
	{ SM_CONF_DEF_MAGIC, NULL }
};

sm_conf_definition_T const duration_suffixes[] =
{
{ SM_CONF_DEF_MAGIC, "s", sm_conf_type_u32_suffix, 1, 60	},
{ SM_CONF_DEF_MAGIC, "m", sm_conf_type_u32_suffix, 60, 60	},
{ SM_CONF_DEF_MAGIC, "h", sm_conf_type_u32_suffix, 60 * 60, 24	},
{ SM_CONF_DEF_MAGIC, "d", sm_conf_type_u32_suffix, 60 * 60 * 24, 7	},
{ SM_CONF_DEF_MAGIC, "w", sm_conf_type_u32_suffix, 60 * 60 * 24 * 7, 365 },
{ SM_CONF_DEF_MAGIC, "y", sm_conf_type_u32_suffix, 60 * 60 * 24 * 365 },

/* Sentinel */
{ SM_CONF_DEF_MAGIC, NULL }
};

static sm_conf_definition_T
qmgr_definitions[] = {
	{ SM_CONF_DEF_MAGIC, "type",
		sm_conf_type_u32,
		offsetof(structure, s_type),
		sizeof(unsigned int),
		NULL,
		SM_CONF_FLAG_STRICTLY_REQUIRED
	},

	{ SM_CONF_DEF_MAGIC, "value",
		sm_conf_type_u32,
		offsetof(structure, s_int_u32),
		sizeof(unsigned int),
		NULL
	},

	{ SM_CONF_DEF_MAGIC, "limit",
		sm_conf_type_u32,
		offsetof(structure, s_limit),
		sizeof(unsigned short),
		NULL,
		0,
		limit_constraints
	},

	/* A number with suffix multipliers. */
	{ SM_CONF_DEF_MAGIC, "duration",
		sm_conf_type_u32,
		offsetof(structure, s_duration),
		sizeof(unsigned long),
		"0",
		0,
		duration_suffixes
	},

{ SM_CONF_DEF_MAGIC, "log", sm_conf_type_section,
	offsetof(structure, s_log),	0,
	NULL, SM_CONF_FLAG_KEEP_DEFAULT, sm_log_spec_defs,
	NULL,	NULL,
	"Specify syslog configuration instead of logging to files"	},

	/* sentinel */
	{ SM_CONF_DEF_MAGIC, NULL }
};

sm_conf_definition_T definitions[] =
{
	{ SM_CONF_DEF_MAGIC, "qmgr",
		sm_conf_type_section,
		0,
		sizeof(structure),
		NULL, SM_CONF_FLAG_ALLOW_ANY, qmgr_definitions
	},

	{ SM_CONF_DEF_MAGIC, "socket",
		sm_conf_type_string,
		offsetof(structure, s_socket),
		0,
		"path/to/socket"
	},

	{ SM_CONF_DEF_MAGIC, "delim",
		sm_conf_type_char,
		offsetof(structure, s_char),
		0,
		"+"
	},

	{ SM_CONF_DEF_MAGIC, "nameserver",
		sm_conf_type_ipv4,
		offsetof(structure, s_ipv4),
		sizeof(ipv4_T),
		"127.0.0.1",
		0
	},

	{ SM_CONF_DEF_MAGIC, "flags",
		sm_conf_type_choice,
		offsetof(structure, s_flags),
		sizeof(uint32_t),
		NULL,
		SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_KEEP_DEFAULT,
		flag_names
	},

	{ SM_CONF_DEF_MAGIC, "aliases",
		sm_conf_type_choice,
		offsetof(structure, s_flags),
		sizeof(uint32_t),
		NULL,
		SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_KEEP_DEFAULT,
		alias_options
	},

	/* sentinel */
	{ SM_CONF_DEF_MAGIC, NULL }
};

static structure s;

static void
print_structure(structure *st)
{
	sm_ret_T ret;
	sm_str_P ipv4s;

	ipv4s = sm_str_new(NULL, 20, 32);
	if (ipv4s == NULL)
	{
		printf("out of memory\n");
		return;
	}
	printf("int_u32: %u\n",		st->s_int_u32);
	printf("limit: %hu\n",		st->s_limit);
	printf("duration: %lu\n",	st->s_duration);
	printf("socket: \"%s\"\n",	st->s_socket);
	printf("flags: %X\n",		st->s_flags);
	printf("char: %c\n",		st->s_char);
	ret = sm_inet_ipv4str(st->s_ipv4, ipv4s);
	if (ret != SM_SUCCESS)
	{
		printf("sm_inet_ipv4str=0x%x\n", ret);
		return;
	}
	printf("nameserver: %s\n",	sm_str_getdata(ipv4s));
}

static int
process(char const *confname, FILE *fp, bool show, bool prt)
{
	sm_conf_T *smc;
	int err;
	char const *name, *kw;
	size_t name_n, kw_n;
#if 0
	sm_conf_iterator_T service_iter;
#endif /* 0 */
	sm_conf_node_T *node, *root;
	char buf[SM_CONF_ERROR_BUFFER_SIZE];
	char const *e = NULL;

	if (((smc = sm_conf_new(confname ? confname : "*stdin*"))) == NULL)
	{
		fprintf(stderr, "error -- sm_conf_new() returns NULL!\n");
		return 1;
	}
	if ((err = sm_conf_read_FILE(smc, confname, fp)) != 0)
	{
		fprintf(stderr, "%s: %s\n",
			confname ? confname : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));

		while ((e = sm_conf_syntax_error(smc, e)) != NULL)
			fprintf(stderr, "%s\n", e);

		sm_conf_destroy(smc);
		return 2;
	}

	s.s_flags = 0x7;
	err = sm_conf_scan(smc, definitions,
		SM_CONF_FLAG_ALLOW_ANY|SM_CONF_FLAG_KEEP_DEFAULT, &s);
	if (err != 0)
	{
#if 0
		fprintf(stderr, "%s: %d\n", confname ? confname : "*stdin*", err);
#endif /* 0 */
		fprintf(stderr, "(while scanning) %s: %s\n",
			confname ? confname : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));

		while ((e = sm_conf_syntax_error(smc, e)) != NULL)
			fprintf(stderr, "%s\n", e);

		sm_conf_destroy(smc);
		return 3;
	}

	if (show)
		print_structure(&s);

	if (prt)
	{
		sm_io_fprintf(smioout, "prt_conf1:\n");
		(void) sm_conf_prt_cnfs(definitions, &s, 0, smioout, 0, NULL);
		sm_io_flush(smioout);
	}
#if 0
	service_iter = NULL;
	while ((err = sm_conf_scan_next(smc, "",
				definitions, 0,
				&service_name, &service_name_n,
				&s, &service_iter)) == 0)
	{
		node = (sm_conf_node_T *) service_iter;
		err = sm_conf_section_keyword(smc, node, &kw, &kw_n);
		if (kw == NULL)
			printf("kw=(none)\n");
		else
			printf("kw=%*s\n", (int) kw_n, kw);
		if (service_name == NULL)
			printf("name=(none)\n");
		else
			printf("name=%*s\n", (int) service_name_n,
				service_name);
	}
#endif /* 0 */

	root = sm_conf_root(smc);
	node = NULL;
	while ((node = sm_conf_section_next_subsection(smc, root,
			NULL, 0,
			NULL, 0,
			node)) != NULL)
	{
		err = sm_conf_section_keyword(smc, node, &kw, &kw_n);
		if (kw == NULL)
			printf("kw=(none)\n");
		else
			printf("kw=%*s\n", (int) kw_n, kw);

		err = sm_conf_section_name(smc, node, &name, &name_n);
		if (name == NULL)
			printf("name=(none)\n");
		else
			printf("name=%*s\n", (int) name_n, name);
		printf("type=%d\n", sm_conf_node_type(smc, node));
		err = sm_conf_get_relative(smc, node, NULL,
				sm_conf_type_section, qmgr_definitions,
				SM_CONF_FLAG_ALLOW_ANY, &s, sizeof(s));
		if (err != 0)
		{
			fprintf(stderr, "%s: %d\n",
				confname ? confname : "*stdin*", err);

#if 0
			fprintf(stderr, "%s: %s\n",
				confname ? confname : "*stdin*",
				sm_conf_strerror(err, buf, sizeof buf));

			while ((e = sm_conf_syntax_error(smc, e)) != NULL)
				fprintf(stderr, "%s\n", e);
#endif /* 0 */
		}
	}

#if 0
	sm_conf_destroy(smc);
#endif /* 0 */

	return 0;
}

int
main(int ac, char **av)
{
	int ai, c, ret;
	bool prt, done;

	done = false;
	prt = false;
	while ((c = getopt(ac, av, "f:l:sv:")) != -1)
	{
		switch (c)
		{
		  case 'f':
			ret = process(optarg, NULL, 0, prt);
			if (ret != 0)
				return ret;
			done = true;
			break;
		  case 'l':
			s.s_limit = atoi(optarg);
			break;
		  case 's':
			prt = true;
			break;
		  case 'v':
			s.s_int_u32 = atoi(optarg);
			break;
		}
	}
	ac -= optind;
	av += optind;
	if (done)
	{
		print_structure(&s);
		if (prt)
		{
			sm_io_fprintf(smioout, "prt_conf:\n");
			(void) sm_conf_prt_cnfs(definitions, &s, 0,
					smioout, 0, NULL);
			sm_io_flush(smioout);
		}
		return 0;
	}

	if (ac == 0)
		return process("*stdin*", stdin, 1, prt);

	for (ai = 0; ai < ac; ai++)
	{
		int ret;

		ret = process(av[ai], NULL, 1, prt);
		if (ret != 0)
			return ret;
	}
	return 0;
}
