#!/bin/sh
# $Id: t-lum-0.sh,v 1.5 2005/05/31 21:11:21 ca Exp $
# Copyright (c) 2004, 2005 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ------------------------------------------------------------
# Description:
# Script to test local user map for RFC2821 addresses.
# Uses smar, t-smar-2, aliases and "local user" map, t-mm, startsmar.sh
# and t-hostname
# ------------------------------------------------------------
#
test -s stop && exit 1
#

# path to programs
P=..
SD=`dirname $0`
SHELL=/bin/sh
OUT=lum-0.out
RES=lum-0.ip
OK=lum-0.ok
SOCK=qmsmar/qmsmar
PIDS=pids
ERRS=0
TPRG=./t-smar-2
ALI=aliases
AM=${ALI}.db
LU=users
LUM=${LU}.db
CONF=lum.conf

# Note: reference to t-hostname! XREF
HOSTNAME=`../libmta/t-hostname -l -V 2>/dev/null`
if test $? -ne 0
then
  echo "t-hostname failed"
  exit 1
fi
if test X"${HOSTNAME}" = X
then
  echo "t-hostname returned empty name"
  exit 1
fi
if echo X"${HOSTNAME}" | grep '\.' >/dev/null
then
  :
else
  echo "t-hostname returned bogus name ${HOSTNAME}"
  exit 1
fi

killit()
{
if test -s ${PIDS}
then
  # stop MTA components
  for i in `cat ${PIDS}`
  do
    kill ${i}
  done
fi
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall smar
fi
#rm mt
exit 1
}

# create map for smar
rm -f ${AM} ${LUM}
../libsmmap/t-mm -F ${LUM} <<EOF
user1:	ok
user2:	rhs
user3:	something
EOF
if test $? -ne 0
then
  echo t-mm failed
  exit 1
fi

# create map for smar
../libsmmap/t-mm <<EOF
postmaster:	user1
EOF
if test $? -ne 0
then
  echo t-mm failed
  exit 1
fi

cat > ${CONF} <<EOF
smar {
 map lum { type=hash; file=${LUM}; }
 local_user_map = lum;
}
EOF

rm -f ${SOCK} ${PIDS}
trap killit 1 2 3

# create mailertable (save old version?)
cat > mt <<EOF
local.dom	lmtp:
${HOSTNAME}	lmtp:
EOF

# start SMAR
${SD}/startsmar.sh -f ${CONF} || exit 1
rm -f ${OUT} ${RES} ${OK}

# check output somehow...
${VALGRIND} ${TPRG} -VVV '<no@local.dom>' > ${OUT} 2>&1
grep '^status=' ${OUT} > ${RES}
cat > ${OK} <<EOF
status=550
EOF
if diff ${OK} ${RES}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  test X"${STOPONERROR}" != X && killit
fi

${VALGRIND} ${TPRG} -VVV '<user1@local.dom>' > ${OUT} 2>&1
grep '^status=' ${OUT} > ${RES}
cat > ${OK} <<EOF
status=0
EOF
if diff ${OK} ${RES}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  test X"${STOPONERROR}" != X && killit
fi

${VALGRIND} ${TPRG} -VVV '<postmaster@local.dom>' > ${OUT} 2>&1
grep '^status=' ${OUT} > ${RES}
cat > ${OK} <<EOF
status=0
EOF
if diff ${OK} ${RES}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  test X"${STOPONERROR}" != X && killit
fi

${VALGRIND} ${TPRG} -VVV '<user2@local.dom>' '<user3@local.dom>' > ${OUT} 2>&1
grep '^status=' ${OUT} > ${RES}
cat > ${OK} <<EOF
status=0
status=0
EOF
if diff ${OK} ${RES}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  test X"${STOPONERROR}" != X && killit
fi

# stop SMAR
for i in `cat ${PIDS}`
do
  kill ${i}
done
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall smar
fi
rm mt

exit ${ERRS}
